// A string class.  Why reuse code when you can roll your own?
//
// $Id: LyStr.cc,v 1.9 1994/07/16 17:42:28 ceder Exp $
// Copyright (C) 1994  Per Cederqvist and Inge Wallin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "LyStr.h"
#include <strstream.h>
#include <memory.h>
#include <string.h>
#include <assert.h>
#include <ctype.h>
#include <stdlib.h>

#define DELETE_CHAR_ARR delete[]

LyStr::LyStr()
{
    data = NULL;
    size = 0;
    used = 0;
    origin = 0;
}

LyStr::LyStr(const char *c_str)
{
    used = ::strlen (c_str);
    size = 2*used;		// Heuristic...
    data = new char [size];
    memcpy(data, c_str, used);
    origin = 0;
}

LyStr::LyStr(void *block,
	     int len)
{
    size = 2 * len;
    data = new char [ size ];
    used = len;
    origin = 0;
    memcpy(data, block, len);
}

LyStr::LyStr(const LyStr &s)
{
    data = new char [ s.size ];
    size = s.size;
    used = s.used;
    origin = 0;
    memcpy(data, s.data+s.origin, used);
}

LyStr::~LyStr()
{
    DELETE_CHAR_ARR data;
}

LyStr &
LyStr::operator << (const LyStr &s)
{
    append(s.data+s.origin, s.used);
    return *this;
}

LyStr &
LyStr::operator << (char c)
{
    append(&c, 1);
    return *this;
}

LyStr &
LyStr::operator << (long num)
{
    char buf[40];
    ostrstream o(buf, 40);
    o << num << '\0';
    append(buf, ::strlen(buf));
    return *this;
}

void
LyStr::append(const char *buffer,
	      long length)
{
    if (origin+used+length <= size)
    {
	if (length)
	    memcpy(data+origin+used, buffer, length);
	used += length;
    }
    else if (used+length <= size)
    {
	for (int i=0; i < used; i++)
	    data[i] = data[origin+i];
	origin = 0;
	if (length)
	    memcpy(data+used, buffer, length);
	used += length;
    }
    else
    {
	int newsize = 2 * (used+length);
	char *newdata = new char [ newsize ];
	if (used)
	    memcpy(newdata, data+origin, used);
	if (length)
	    memcpy(newdata+used, buffer, length);
	origin = 0;
	if (data)
	    DELETE_CHAR_ARR data;
	data = newdata;
	size = newsize;
	used += length;
    }
}

char &
LyStr::operator [] (long index)
{
    assert(index >= 0 && index < used);
    return data[origin+index];
}

LyStr & 
LyStr::operator = (const LyStr &s)
{
    if (size < s.used)
    {
	if (data)
	    delete [] data;
	size = 2 * s.used;
	data = new char [ size ];
    }
    if (s.used)
        memcpy(data, s.data+s.origin, s.used);
    origin = 0;
    used = s.used;
    return *this;
}

long
LyStr::strlen() const
{
    return used;
}

void
LyStr::remove_first(long no)
{
    assert (no <= used);
    origin += no;
    used -= no;
}

void
LyStr::remove_last(long no)
{
    assert (no <= used);
    used -= no;
}

void
LyStr::clear()
{
    origin = 0;
    used = 0;
}

void
LyStr::skipspace()
{
    while (strlen() > 0 && operator[](0) == ' ')
	remove_first(1);
}

char *
LyStr::malloc_copy(long start, long wanted_size) const
{
    assert(start >= 0);
    assert(wanted_size >= 0);

    if (wanted_size > used - start)
	wanted_size = used - start;

    char *copy = (char*) malloc(wanted_size+1);
    if (copy != NULL)
    {
	memcpy(copy, data+origin+start, wanted_size);
	copy[wanted_size] = '\0';
    }
    return copy;
}

LyStr
LyStr::substr(long sub) const
{
    if (sub > used)
	return *this;

    LyStr ret(data+origin, sub);
    return ret;
}

ostream &
operator << (ostream &os, const LyStr &s)
{
    for (int i=s.origin; i < s.used+s.origin; i++)
	os << s.data[i];

    return os;
}
