// A string class.  Why reuse code when you can roll your own?
//
// $Id: LyStr.h,v 1.10 1994/07/16 17:42:32 ceder Exp $
// Copyright (C) 1994  Per Cederqvist and Inge Wallin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#ifndef LyStr__
#define LyStr__

#include <iostream.h>
#include <limits.h>

class Connection;

// A simple string package. Why is libg++ so limited?
class LyStr {
  public:
    LyStr();
    LyStr(const char *c_str);
    LyStr(void *block, int len);
    LyStr(const LyStr &);
    ~LyStr();
    LyStr & operator = (const LyStr &);

    // operator << (...)  appends to the string
    LyStr &operator << (const LyStr &str);
    LyStr &operator << (char c);
    LyStr &operator << (long num); // Append decimal number
    void append(const char *buffer, long length);
    char & operator [] (long index);
    long strlen() const;
    void clear();
    void remove_first(long no);	// Remove first NO chars from the string.
    void remove_last(long no);	// Remove last NO chars from the string.
    void skipspace();		// Remove leading whitespace

    // Create a malloc'd copy of the string. If start and length are
    // given, a substring starting at start and length characters long
    // is returned. The malloced copy is always nul-terminated. The
    // nul is not counted in length.
    char *malloc_copy(long start=0, long length=LONG_MAX) const;

    // Create a new string, consisting of the first SIZE characters
    // from the string.
    LyStr substr(long size) const;

    friend ostream &operator <<(ostream &, const LyStr &);

  protected:
    char *data;			// size chars big
    long size;
    long used;			// Number of chars used.
    long origin;		// First char used.
};
#endif
