// Fetch-on-demand conference object.
//
// $Id: conference.cc,v 1.13 1994/07/16 17:43:37 ceder Exp $
// Copyright (C) 1994  Per Cederqvist and Inge Wallin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include <assert.h>
#include <stdlib.h>
#include <iostream.h>
#include "conference.h"
#include "conf-cache.h"


// ================================================================
// ================================================================
//                            Class Conference


// ================================================================ 
//                    Constructors and Destructors                  


Conference::Conference(Conf_no confno, Conf_cache * cc)
    : cno(confno),
      cache(cc)
{
}

 
Conference::Conference(const Conference &conf)
    : cno(conf.cno),
      cache(conf.cache)
{
}


Conference::~Conference()
{
}


// ================================================================ 
//

const Conference & 
Conference::operator = (const Conference &)
{
    cerr << "Operator= for class Conference is not yet implemented\n";
    abort();
}


Status
Conference::prefetch() const
{
    return cache->prefetch(cno);
}


Bool
Conference::valid() const
{
    return cache->get(cno).valid();
}

int
Conference::errno() const
{
    return cache->get(cno).errno();
}

Bool
Conference::present() const
{
    return cache->present(cno);
}

Conf_no
Conference::conf_no() const
{
    return cno;
}

Connection *
Conference::connection() const
{	
    return cache->connection();
}

Pers_no 
Conference::creator() const
{
    assert(valid());
    return cache->get(cno).creator();
}

struct tm 
Conference::creation_time() const
{
    assert(valid());
    return cache->get(cno).creation_time();
}

Text_no 
Conference::presentation() const
{
    assert(valid());
    return cache->get(cno).presentation();
}

Conf_no 
Conference::supervisor() const
{
    assert(valid());
    return cache->get(cno).supervisor();
}

Conf_no 
Conference::permitted_submitters() const
{
    assert(valid());
    return cache->get(cno).permitted_submitters();
}

Conf_no 
Conference::super_conf() const
{
    assert(valid());
    return cache->get(cno).super_conf();
}

Conf_type 
Conference::type() const
{
    assert(valid());
    return cache->get(cno).type();
}

struct tm 
Conference::last_written() const
{
    assert(valid());
    return cache->get(cno).last_written();
}

Text_no 
Conference::msg_of_day() const
{
    assert(valid());
    return cache->get(cno).msg_of_day();
}

Garb_nice 
Conference::nice() const
{
    assert(valid());
    return cache->get(cno).nice();
}

LyStr 
Conference::name() const
{
    assert(valid());
    return cache->get(cno).name();
}

unsigned short 
Conference::no_of_members() const
{
    assert(valid());
    return cache->get(cno).no_of_members();
}

Local_text_no 
Conference::first_local_no() const
{
    assert(valid());
    return cache->get(cno).first_local_no();
}

unsigned long 
Conference::no_of_texts() const
{
    assert(valid());
    return cache->get(cno).no_of_texts();
}


// ================================================================
// ================================================================
//                            Class Conferences


// ================================================================ 
//                    Constructors and Destructors                  


Conferences::Conferences()
  : conn(NULL),
    conf_cache(NULL)
{
}


Conferences::Conferences(Connection *co)
  : conn(co),
    conf_cache(new Conf_cache(co))
{
}
 

Conferences::Conferences(const Conferences &conf)
    : conn(conf.conn)
{
    // Varning! Referensr{knande conf_cache eller liknande beh|vs!
    abort();
}


Conferences::~Conferences()
{
    if (conf_cache)
	delete conf_cache;
}


// ================================================================ 
//

void
Conferences::set_connection(Connection *co)
{
    assert(conn == NULL);
    conn = co;
    conf_cache = new Conf_cache(conn);
}

Connection *
Conferences::connection() const
{
    return conn;
}

const Conferences & 
Conferences::operator = (const Conferences &)
{
    cerr << "Operator= for class Conferences is not yet implemented\n";
    abort();
}


Conference
Conferences::operator [](Conf_no confno) const
{
    assert(conf_cache != NULL);
    return Conference(confno, conf_cache);
}

Conf_cache * 
Conferences::read_conf_cache() const
{
    return conf_cache;
}

