// Handle a membership in a conference.
//
// $Id: membership.cc,v 1.8 1994/07/16 17:44:41 ceder Exp $
// Copyright (C) 1994  Per Cederqvist and Inge Wallin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "prot-a-LyStr.h"
#include "membership.h"
#include "time-utils.h"


Membership::Membership()
: // last_time_read(),
      m_conf_no(0),
      m_priority(0),
      m_last_text_read(0),
      read_texts()
{
}

Membership::Membership(prot_a_LyStr &prot_a_str)
{
    int   arrsize;
    int   i;

    m_last_time_read = parse_time(prot_a_str);
    m_conf_no = prot_a_str.stol();
    m_priority = prot_a_str.stol();
    m_last_text_read = prot_a_str.stol();

    arrsize = prot_a_str.parse_array_start();
    for (i = 0; i < arrsize; ++i) 
    {
	read_texts.append(prot_a_str.stol());
    }

    prot_a_str.parse_array_end(arrsize);
}


Membership::Membership(const Membership &m)
    : m_conf_no(m.m_conf_no),
      m_priority(m.m_priority),
      m_last_time_read(m.m_last_time_read),
      m_last_text_read(m.m_last_text_read),
      read_texts(m.read_texts)
{
}

const Membership &
Membership::operator =(const Membership &m)
{
    m_conf_no = m.m_conf_no;
    m_priority = m.m_priority;
    m_last_time_read = m.m_last_time_read;
    m_last_text_read = m.m_last_text_read;
    read_texts = m.read_texts;
    return *this;
}


// ================================================================
//                            Access methods


Bool
Membership::is_read(Local_text_no lto) const
{
    if (lto <= m_last_text_read)
	return TRUE;

    // Casts due to non-const declaration of first() and next() in libg++
    for  (Pix i = ((DLList<Local_text_no>)read_texts).first(); 
	  i != 0; ((DLList<Local_text_no>)read_texts).next(i)) {
	if (((DLList<Local_text_no>)read_texts)(i) == lto)
	    return TRUE;
    }

    return FALSE;
}


Local_text_no
Membership::last_text_read() const
{
    return m_last_text_read; 
}

long	   
Membership::num_read_after_last_text_read() const
{
    return ((Membership *)this)->read_texts.length();
}
