// Handle information about conferences with unread texts.
//
// $Id: unread-confs.cc,v 1.16 1994/07/16 17:47:37 ceder Exp $
// Copyright (C) 1994  Per Cederqvist and Inge Wallin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include <iostream.h>
#include <stdlib.h>

#include "unread-confs.h"
#include "connection.h"
#include "get_unread_confs_q.h"
#include "pepsi_q.h"

// ================================================================ 
//                    Constructors and Destructors                  


Unread_confs::Unread_confs(Session *sess, Pers_no the_reader)
    : session(sess), conferences(sess->conferences()), 
      persons(sess->persons()), maps(sess->maps()), texts(sess->texts()),
      unread_confs(), 
      reader(the_reader), read_confs(), current_working_conf(0),
      ucq(NULL), unread_confs_has_been_fetched(FALSE)
{
}
 

Unread_confs::Unread_confs(const Unread_confs &cl)
    : session(cl.session), conferences(cl.conferences), persons(cl.persons), 
      maps(cl.maps), texts(cl.texts), unread_confs(cl.unread_confs), 
      reader(cl.reader), read_confs(cl.read_confs),
      current_working_conf(cl.current_working_conf), ucq(cl.ucq),
      unread_confs_has_been_fetched(cl.unread_confs_has_been_fetched)
{
}


Unread_confs::~Unread_confs()
{
    delete ucq;
}


// ================================================================ 
//

//const Unread_confs & 
//Unread_confs::operator = (const Unread_confs &)
//{
//    cerr << "Operator= for class Unread_confs is not yet implemented\n";
//    return *this;
//}


// ================================================================
//

int
Unread_confs::prefetch_map(int required_texts,
			int text_in_confs,
			int confs,
			int max_question,
			int min_question)
{
    if (!unread_confs_has_been_fetched)
    {
	ucq = new get_unread_confs_question(conferences.connection(), reader);
	unread_confs_has_been_fetched = TRUE;
    }

    if (ucq != NULL && ucq->receive() == st_pending)
	return 1;

    if (ucq != NULL)
    {
	if (ucq->receive() == st_error)
	   return -1;

	assert(unread_confs.empty() && read_confs.empty());

	Array<Conf_no> new_unread_confs = ucq->result();
	delete ucq;
	ucq = NULL;

	for (int conf_ix = 0; conf_ix < new_unread_confs.size(); conf_ix++)
	{
	    add_conference(new_unread_confs[conf_ix], TRUE);
	}
    }

    int questions = 0;

    // Step 1: make sure we know about at least REQUIRED_TEXTS texts.

    for (Pix ix = unread_confs.first();
	 ix != 0 && required_texts > 0 && questions < max_question;
	 unread_confs.next(ix))
    {
	int nq = unread_confs(ix)->prefetch_map(required_texts);
	int nt = unread_confs(ix)->min_num_unread();

	required_texts -= nt;
	if (nq > 0)
	    questions += nq;
	
	if (nq == 0 && nt == 0)
	{     
	    // There is nothing to read in this conference. Remove it
	    // from unread_confs.
	    read_confs.append(unread_confs(ix));
	    unread_confs.del(ix, -1);
	}
    }
    
    // Step 2: Know about at least TEXT_IN_CONFS unread texts in at
    // least CONF conferences.

    for (ix = unread_confs.first();
	 ix != 0 && confs > 0 && questions < max_question;
	 unread_confs.next(ix))
    {
	int nq = unread_confs(ix)->prefetch_map(text_in_confs);
	int nt = unread_confs(ix)->min_num_unread();

	confs--;
	
	if (nq > 0)
	    questions += nq;
	
	if (nq == 0 && nt == 0)
	{     
	    // There is nothing to read in this conference. Remove it
	    // from unread_confs.
	    read_confs.append(unread_confs(ix));
	    unread_confs.del(ix, -1);
	}
    }

    // Step 3: Issue at least MIN_QUESTION questions.

    for (ix = unread_confs.first();
	 ix != 0 && questions < min_question;
	 unread_confs.next(ix))
    {
	int nq = unread_confs(ix)->prefetch_map(999999); // FIXME+++
	int nt = unread_confs(ix)->min_num_unread();

	if (nq > 0)
	    questions += nq;
	
	if (nq == 0 && nt == 0)
	{     
	    // There is nothing to read in this conference. Remove it
	    // from unread_confs.
	    read_confs.append(unread_confs(ix));
	    unread_confs.del(ix, -1);
	}
    }
    
    return questions;
}

int
Unread_confs::prefetch(int required_texts,
		    int text_in_confs,
		    int confs,
		    int max_question,
		    int min_question)
{
    int questions = prefetch_map(required_texts, text_in_confs, confs,
				 max_question, min_question);

    for (Pix ix = unread_confs.first();
	 ix != 0 && required_texts > 0 && questions < max_question; 
	 unread_confs.next(ix))
    {
	Read_order *r = unread_confs(ix);
	int nq = r->prefetch(required_texts, max_question-questions, 
			     min_question-questions);
	int nt = r->min_num_unread();

	required_texts -= nt;
	if (nq > 0)
	    questions += nq;
    }

    for (ix = unread_confs.first();
	 ix != 0 && confs > 0 && questions < max_question;
	 unread_confs.next(ix))
    {
	Read_order *r = unread_confs(ix);
	int nq = r->prefetch(text_in_confs, max_question-questions, 
			     min_question-questions);
	confs--;
	
	if (nq > 0)
	    questions += nq;
    }

    for (ix = unread_confs.first();
	 ix != 0 && questions < min_question;
	 unread_confs.next(ix))
    {
	Read_order *r = unread_confs(ix);
	int nq = r->prefetch(999999, /// FIXME+++
			     max_question-questions, 
			     min_question-questions);
	if (nq > 0)
	    questions += nq;
    }

    return questions;
}

Conf_no
Unread_confs::goto_next_conf()
{
    while (prefetch_map(1,0,0,10,0) > 0)
	if (Connection::drain(100) == st_error)
	    return 0;

    if (unread_confs.empty())
	return current_working_conf;

    // Unread_confs::prefetch should remove any empty Read_order.
    assert (unread_confs.front()->empty() == FALSE);

    // Are we leaving this conference before everything is unread? If
    // so, move it to the tail of unread conferences.
    if (current_working_conf == unread_confs.front()->confno())
	unread_confs.append(unread_confs.remove_front()); 

    while (prefetch_map(1,0,0,10,0) > 0)
	if (Connection::drain(100) == st_error)
	    return 0;

    assert (unread_confs.front()->empty() == FALSE);

    if (current_working_conf != unread_confs.front()->confno())
    {
	current_working_conf = unread_confs.front()->confno();
	pepsi_question q(conferences.connection(), current_working_conf);
	q.receive();		// +++ Ignore this question.
    }

    return current_working_conf;
}

Status
Unread_confs::goto_conf(Conf_no c)
{
    Read_order *ro = NULL;

    Pix ix;

    for (ix = unread_confs.first(); ix != 0; unread_confs.next(ix))
	if (unread_confs(ix)->confno() == c)
	{
	    ro = unread_confs(ix);
	    unread_confs.del(ix);
	    break;
	}

    if (ro == NULL)
	for (ix = read_confs.first(); ix != 0; read_confs.next(ix))
	    if (read_confs(ix)->confno() == c)
	    {
		ro = read_confs(ix);
		read_confs.del(ix);
		break;
	    }

    if (ro == NULL)
	ro = new Read_order(session, c , reader);

    pepsi_question q(conferences.connection(), c);
    // +++ We could relatively safely ignore this question, except
    // when we had to create a new Read_order.
    if (q.receive() == st_ok)
    {
	unread_confs.prepend(ro);
	current_working_conf = c;
    }
    else
    {
	// The user was not a member in this conference.
	delete ro;
    }
    return q.receive();
}

Prompt
Unread_confs::next_prompt()
{
    Prompt   prompt;

    while (prefetch_map(1,0,0,10,0) > 0)
	if (Connection::drain(100) == st_error)
	    return PROMPT_ERROR;

    if (unread_confs.empty())
	return NO_MORE_TEXT;

    if (current_working_conf != unread_confs.front()->confno())
	return NEXT_CONF;

    prompt = unread_confs.front()->next_prompt();
    assert(prompt != NO_MORE_TEXT && prompt != NEXT_CONF);
    return prompt;
}


Text_no
Unread_confs::next_text(Bool remove)
{
    while (prefetch_map(1,0,0,10,0) > 0)
	if (Connection::drain(100) == st_error)
	    return 0;

    if (unread_confs.empty())
	return 0;

    return unread_confs.front()->next_text(remove);
}

Conf_no
Unread_confs::current_conf()
{
    return current_working_conf;
}

Array<Unread_info>
Unread_confs::unread_texts()
{
    Array<Unread_info>   arr(unread_confs.length());
    int                  index = 0;

    for (Pix ix = unread_confs.first(); ix != 0; unread_confs.next(ix)) 
    {
	arr[index].conf = unread_confs(ix)->confno();
	arr[index].min_unread = unread_confs(ix)->min_num_unread();
	arr[index].max_unread = unread_confs(ix)->max_num_unread();
	index++;
    }

    return arr;
}

long
Unread_confs::min_unread_texts_in_current()
{
    if (unread_confs.empty())
	return 0;
    return unread_confs.front()->min_num_unread();

}

long
Unread_confs::max_unread_texts_in_current()
{
    if (unread_confs.empty())
	return 0;
    return unread_confs.front()->max_num_unread();
}

void
Unread_confs::add_conference(Conf_no cno, Bool last)
{
    // BUG: There should be a way to insert the new conference in the
    // correct place. We do not know the order of the conferences. Big
    // lose.+++

    // +++BUG 2: The conference might already be present in
    // unread_confs or read_confs. We should check for that before
    // creating the new Read_order.

    Read_order  * read_order;

    read_order = new Read_order(session, cno, reader);
    if (last)
	unread_confs.append(read_order);
    else
	unread_confs.prepend(read_order);
    // Note: If there were no texts in this conference, it will be
    // moved to read_confs as soon as we start to prefetch from it.
}
