// Startup and main loop of nilkom.
//
// $Id: tcl-main.cc,v 1.16 1994/07/16 16:39:24 inge Exp $
// Copyright (C) 1994  Per Cederqvist and Inge Wallin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


#include <signal.h>
extern "C" {
#include <termios.h>
}

#include <sys/termios.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <fcntl.h>

#include "misc.h"
#include "tcl-glue.h"
#include "connection.h"
#include "userio.h"
#include "tcl-main.h"

struct sigaction default_abort_act;

LyStr unread_command_string = "";
IscSession *stdincb = NULL;

void
main_loop(Tcl_Interp *interp)
{
    int res;

    while (1)
    {
	while (unread_command_string.strlen() > 0)
	{
	    char buf[20];
	    sprintf(buf, "keypress %d", unread_command_string[0]);
	    unread_command_string.remove_first(1);
	    if ((res = Tcl_Eval(interp, buf)) != TCL_OK)
	    {
		printf("TCL Error ");
		if (res != TCL_ERROR)
		    printf("%d", res);
		if (interp->result[0] != '\0')
		    printf(": %s", interp->result);
		putchar('\n');
		return;
	    }
		
	    if (interp->result[0] != '0' || interp->result[1] != '\0')
		return;
	}

	cout.flush();
	
	IscEvent *ev = Connection::get_event(1000);
	if (ev == &Connection::error_event)
	{
	    cerr << "Connection lost" << endl;
	    return;
	}

	if (ev != NULL && ev->event != ISC_EVENT_TIMEOUT)
	{
	    assert (ev->session == stdincb);
	    switch(ev->event)
	    {
	    case ISC_EVENT_ERROR:
		abort();
	    case ISC_EVENT_MESSAGE:
		unread_command_string << LyStr(ev->msg->buffer,
					       ev->msg->length);
		break;
	    default:
		abort();
	    }
	}
	else if (ev != NULL)
	{
	    assert(ev->event == ISC_EVENT_TIMEOUT);
	    char buf[10];
	    sprintf(buf, "timeout");
	    if ((res = Tcl_Eval(interp, buf)) != TCL_OK)
	    {
		printf("TCL Error ");
		if (res != TCL_ERROR)
		    printf("%d", res);
		if (interp->result[0] != '\0')
		    printf(": %s", interp->result);
		putchar('\n');
		return;
	    }

	    if (interp->result[0] != '0' || interp->result[1] != '\0')
		return;
	}

	if (ev != NULL)
	    isc_dispose(ev);
    }
    /*NOTREACHED*/
    abort();
}

static IscMaster *    
init_isc()
{
    IscMaster *mcb;
    IscConfig config;
    
    config.version = 1005;
    config.master.version = 1001;
    config.master.memfn.alloc = NULL; // Use default allocators.
    config.master.memfn.realloc = NULL;
    config.master.memfn.free = NULL;
    config.master.abortfn = NULL; /* Use default abort function. */
#ifdef ISC_0_98_4
    config.session.version = 1002;
#else
    config.session.version = 1001;
#endif
    config.session.max.msgsize = -1; /* Use default sizes. */
    config.session.max.queuedsize = -1;
    config.session.max.dequeuelen = -1;
    config.session.max.openretries = -1;
    config.session.max.backlog = -1;
#ifdef ISC_0_98_4
    config.session.flags.txempty = 0;
#endif
    
    mcb  = isc_initialize(&config);
    if ( mcb == NULL )
    {
	cerr << "can't isc_initialize()\n";
	exit(1);
    }

    return mcb;
}

struct termios origmode;
int oldflags;

void
restore_kbd_and_exit()
{
    if (tcsetattr(0, TCSADRAIN, &origmode)!=0)
    {
	perror("tcsetattr failed");
	sleep(1);
    }

    if (fcntl(0, F_SETFL, oldflags) != 0)
    {
	perror("fcntl F_SETFL failed");
	sleep(1);
    }

    exit(0);
}

void
restore_kbd_and_abort()
{
    if (tcsetattr(0, TCSADRAIN, &origmode)!=0)
    {
	perror("tcsetattr failed");
	sleep(1);
    }

    if (fcntl(0, F_SETFL, oldflags) != 0)
    {
	perror("fcntl F_SETFL failed");
	sleep(1);
    }

    abort();
    fputs("I survived an abort!\n", stderr);
    sigaction(SIGABRT, &default_abort_act, NULL);
    abort();
    fputs("I survived two aborts!\n", stderr);
    exit(1);
}


int
main(int argc,
     char **argv)
{
    struct termios newmode;
    int result;

    Tcl_Interp *motor = Tcl_CreateInterp();
    mcb_for_isc = init_isc();

    // Set up stdin in non-blocking mode.
    if ((oldflags = fcntl(0, F_GETFL, 0)) == -1)
    {
	perror("fcntl F_GETFL failed");
	exit(1);
    }
    if (tcgetattr(0, &origmode) != 0)
    {
	perror("tcgetattr failed");
	exit(1);
    }

    // Trap all signals
    struct sigaction act;
    sigemptyset(&act.sa_mask);
    act.sa_flags = 0;
    act.sa_handler = restore_kbd_and_exit;
    sigaction(SIGHUP, &act, NULL);
    sigaction(SIGINT, &act, NULL);
    sigaction(SIGTERM, &act, NULL);
    act.sa_handler = restore_kbd_and_abort;
    act.sa_flags = SA_RESETHAND;
    sigaction(SIGABRT, &act, &default_abort_act);
    sigaction(SIGQUIT, &act, NULL);
    sigaction(SIGBUS, &act, NULL);
    sigaction(SIGSEGV, &act, NULL);
    act.sa_handler = SIG_IGN;
    sigaction(SIGWINCH, &act, NULL);
    
    newmode = origmode;
    newmode.c_lflag &= ~(IEXTEN | ECHO | ISIG | ICANON);
    if (tcsetattr(0, TCSADRAIN, &newmode) != 0)
    {
	perror("tcsetattr failed");
	restore_kbd_and_exit();
    }

    stdincb = isc_openfd(mcb_for_isc, 0);
    
    kom_define_commands(motor);
    define_userio_commands(motor);

    char *initscript = "source $kom_library/nilkom.tcl";

    if (argc == 2 && !strcmp(argv[1], "--www"))
	initscript = "source $kom_library/www.tcl";

    // Set argc/argv for the TCL program
    char   buf[43];
    sprintf(buf, "%d", argc);
    Tcl_SetVar(motor, "argc", buf, TCL_GLOBAL_ONLY);
    if (argc == 0)
	Tcl_SetVar(motor, "argv", "", TCL_GLOBAL_ONLY);
    else 
    {
	for (int i = 0; i < argc; ++i) 
	    Tcl_SetVar(motor, "argv", argv[i],
		       TCL_GLOBAL_ONLY | TCL_APPEND_VALUE | TCL_LIST_ELEMENT);
    }

    strcpy(buf, initscript);	// TCL wants to write in this string!
    result = Tcl_Eval(motor, buf);
    if (result != TCL_OK) 
    {
	fprintf(stderr, "%s\n", motor->result);
	restore_kbd_and_exit();
    }

    main_loop(motor);
    restore_kbd_and_exit();
}
