# Commands found under the "Read" menu in tkom
#
# $Id: tkom_commands_read.tcl,v 1.13 1994/07/16 16:39:50 inge Exp $
# Copyright (C) 1994  Per Cederqvist and Inge Wallin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


# ================================================================
#              Command: Show number of unread texts


proc show_unread_command {} {
    if {![start_of_command show_unread_command]} {
	return
    }

    #
    # Start by prefetching the information we need
    #
    set unread_confs [llength [kom_unread unread_texts]]
    set prefetch_calls 0
    while {[kom_unread prefetch_map 0 20 $unread_confs 40 0] > 0} {
        if {$prefetch_calls == 0} {
            show_message "Jag mste hmta lite information frn servern\n"
            show_message "Vnta ett tag.."
        }
        show_message "."
	update
	incr prefetch_calls 1
    }
    if {$prefetch_calls > 0} {
        show_message "klart\n"
    }
    kom_unread prefetch_map 0 0 0 10 10

    #
    # Ok, time to show how many unread texts we have.
    #
    set mintotal 0
    set maxtotal 0
    show_message "Du har olsta inlgg i fljande mten:\n"
    foreach conf [kom_unread unread_texts] {
        set confno [lindex $conf 0]
        set min    [lindex $conf 1]
        set max    [lindex $conf 2]

	if {$max == $min} {
	    if {$max > 0} {
                show_message [format "     %6d %s\n" $min \
                                [kom_conference name $confno]]
            }
	} else {
                show_message [format "%4d-%6d %s\n" $min $max \
                                [kom_conference name $confno]]
	}
	incr mintotal $min
        incr maxtotal $max
    }
    if {$mintotal == $maxtotal} {
        show_message "\nTotalt $mintotal olsta\n\n"
    } else {
        show_message "\nTotalt mellan $mintotal och $maxtotal olsta\n"
    }

    end_of_command
}


# ================================================================
#                      Command: Goto conference


# This variable is 1 when there are alternatives to choose from
# in the listbox 'conflist'.
set conflist_contains_confs   0


proc goto_conf_window {w} {
    global romanfont
    global boldfont

#    transient_window $w . "G till mte"
    butwin $w -buttons {{"Vlj"} {"Avbryt"}}
    wm title $w "G till mte"

    # The selector frame
    frame $w.top 
    pack $w.top -side top -fill both
    listbox $w.top.conflist -relief raised -setgrid true \
	-yscrollcommand "$w.top.listscroll set"
    scrollbar $w.top.listscroll -relief flat -command "$w.top.conflist yview"
    pack $w.top.conflist -side left -expand TRUE -fill both
    pack $w.top.listscroll -side right -fill y

#    labeled_entry $w.conf "Mte:" 6
    lblentry $w.conf -lbltext "Mte" -lblwidth 6
    pack $w.conf -side top -fill x 

#    ok_cancel_buttons $w "Vlj"
#    transient_window_center $w

#    labeled_entry_bind $w.conf <Return> "stop_waiting_for_window ok"
#    $w.conf bind <Return> "stop_waiting_for_window ok"
    bind $w.top.conflist <Double-1>  "goto_conf_click_on_conf $w %y"
}


proc goto_conf_window_destroy {w} {
    $w.conf delete
    $w delete
}

proc goto_conf_click_on_conf {w y} {
    global conflist_contains_confs

    if {$conflist_contains_confs} {
#	labeled_entry_put $w.conf [$w.top.conflist get \
#				      [$w.top.conflist nearest $y]]
        $w.conf put [$w.top.conflist get [$w.top.conflist nearest $y]]
    }
}


proc goto_conf_command {{conf {}}} {
    global conflist_contains_confs
    global passwd_string

    if {![start_of_command goto_conf_command]} {
	return
    }

    goto_conf_window .goto_conf
    tkwait visibility .goto_conf
    if {$conf != ""} {
#	labeled_entry_put $w.conf $conf
	$w.conf put $conf
    }
    set conflist_contains_confs 0

    while {1} {
#	labeled_entry_focus .goto_conf.conf
	.goto_conf.conf focus
        if {[.goto_conf wait] == 0} {
#	    set conf_to_go_to [labeled_entry_get .goto_conf.conf]
	    set conf_to_go_to [.goto_conf.conf get]
            set matches [kom_session lookup_name $conf_to_go_to]
            .goto_conf.top.conflist delete 0 end

            set conflist_contains_confs 0
            if {[llength $matches] == 0} {
                .goto_conf.top.conflist delete 0 end
                .goto_conf.top.conflist insert end "Ingen trff p namnet"
                set conflist_contains_confs 0

            } elseif {[llength $matches] == 1} {
                # Only 1 conf matched ==> Try to go there.
                set confno [microconf confno [lindex $matches 0]]

                if {[safe_eval {kom_unread goto_conf $confno; format TRUE} \
			       {{{KOM} {return FALSE}}}]} \
		{
		    show_current_conf $confno
                    set current_conf  $confno
                    destroy .goto_conf
                    end_of_command
                    return
                } else {
                    .goto_conf.top.conflist delete 0 end
                    .goto_conf.top.conflist insert end "Du fick inte g dit."
                    set conflist_contains_confs 0
                }

            } elseif {[llength $matches] > 1} {
                set i 0
                while {$i < [llength $matches]} {
                    set match [lindex $matches $i]
                    if {![microconf is-letterbox $match]} {
                        .goto_conf.top.conflist insert end \
			    [kom_conference name [microconf confno $match]]
                    }
                    incr i
                }
                set conflist_contains_confs 1
            }
        } else {
	    goto_conf_window_destroy .goto_conf
#            destroy .goto_conf
	    end_of_command
            return
        }
    }
}
